#include <stdio.h>  //fseek fread
#include <fcntl.h>
#include <unistd.h>
#include <stdint.h>
#include <stdlib.h> //memset

/****************************** Bitmap standard information*************************************/
/*Bitmap file header   14bit*/
typedef struct BMP_FILE_HEADER {
    uint16_t bType;                 //File identifier
    uint32_t bSize;                //The size of the file
    uint16_t bReserved1;            //Reserved value, must be set to 0
    uint16_t bReserved2;            //Reserved value, must be set to 0
    uint32_t bOffset;              //The offset from the beginning of the file header to the beginning of the image data bit
} __attribute__ ((packed)) BMPFILEHEADER;    // 14bit

/*Bitmap information header  40bit*/
typedef struct BMP_INFO {
    uint32_t bInfoSize;            //The size of the header
    uint32_t bWidth;               //The width of the image
    uint32_t bHeight;              //The height of the image
    uint16_t bPlanes;               //The number of planes in the image
    uint16_t bBitCount;             //The number of bits per pixel
    uint32_t bCompression;         //Compression type
    uint32_t bmpImageSize;         //The size of the image, in bytes
    uint32_t bXPelsPerMeter;       //Horizontal resolution
    uint32_t bYPelsPerMeter;       //Vertical resolution
    uint32_t bClrUsed;             //The number of colors used
    uint32_t bClrImportant;        //The number of important colors
} __attribute__ ((packed)) BMPINF;

/*Color table: palette */
typedef struct RGB_QUAD {
    uint8_t rgbBlue;               //Blue intensity
    uint8_t rgbGreen;              //Green strength
    uint8_t rgbRed;                //Red intensity
    uint8_t rgbReversed;           //Reserved value
} __attribute__ ((packed)) RGBQUAD;


#define RGB(r,g,b)  (((r >> 3) << 11) | ((g >> 2) << 5) | (b >> 3))

static void Paint_SetPixel(uint16_t Xpoint, uint16_t Ypoint, uint16_t Color);
static uint16_t image[240 * 240];

int main(int argc, char **argv)
{
    FILE *fp;                     //Define a file pointer 
    BMPFILEHEADER bmpFileHeader;  //Define a bmp file header structure
    BMPINF bmpInfoHeader;         //Define a bmp bitmap header structure
    int opt;
    char *infile, *outfile;

    while((opt = getopt(argc, argv, "i:o:")) != -1) {
        if(opt == 'i') {
            infile = optarg;
        } else if(opt == 'o') {
            outfile = optarg;
        }
    }

    // Binary file open
    if((fp = fopen(infile, "rb")) == NULL) { // fp = 0x00426aa0
        printf("Cann't open the file!\n");
        return 0;
    }

    // Set the file pointer from the beginning
    fseek(fp, 0, SEEK_SET); // fp = 0x00426aa0
    fread(&bmpFileHeader, sizeof(BMPFILEHEADER), 1, fp);//	sizeof(BMPFILEHEADER) must be 14,
    fread(&bmpInfoHeader, sizeof(BMPINF), 1, fp);
    
    int row, col;
    short data;
    RGBQUAD rgb;
    int len = bmpInfoHeader.bBitCount / 8;    //RGB888,one 3 byte = 1 bitbmp

    // get bmp data and show
    fseek(fp, bmpFileHeader.bOffset, SEEK_SET);
    for(row = 0; row < bmpInfoHeader.bHeight; row++) {
        for(col = 0; col < bmpInfoHeader.bWidth; col++) {
            if(fread((char *)&rgb, 1, len, fp) != len){
                perror("get bmpdata:\r\n");
                break;
            }
            data = RGB((rgb.rgbRed), (rgb.rgbGreen), (rgb.rgbBlue));
            Paint_SetPixel(col, bmpInfoHeader.bHeight - row - 1, data);
        }
    }
    fclose(fp);

    if((fp = fopen(outfile, "w")) == NULL) {
        printf("Cann't open the file!\n");
        return 0;
    }
    fprintf(fp, "#ifndef __BITMAP_H__\n");
    fprintf(fp, "#define __BITMAP_H__\n\n");
    fprintf(fp, "uint16_t bitmap[240 * 240] = {\n");
    for(uint32_t i = 1; i < 240 * 240; i++) {
        fprintf(fp, "0x%04x,", image[i - 1]);
        if(i % 16 == 0) {
            fprintf(fp, "\n");
        }
    }
    fprintf(fp, "0x%04x};\n\n", image[240 * 240 - 1]);
    fprintf(fp, "#endif\n\n");
    fclose(fp);

    return 0;
}

static void Paint_SetPixel(uint16_t X, uint16_t Y, uint16_t Color)
{
    Color = ((Color <<8 ) & 0xff00) | (Color >> 8);
    image[X + Y * 240] = Color;
}
